/*
* Copyright (C) 2006 Roberto Alsina <ralsina@kde.org>
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either
* version 2 of the License, or (at your option) any later
* version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software Foundation,
* Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*
*/

#include <stdio.h>
#include <stdlib.h>
#include "bstrlib.h"
#include <sys/types.h>
#include <unistd.h>

#include <relay.h>

/*

Plugin to connect to relayd, a connection throttle mechanism.
This works on HELO, and checks TCPREMOTEIP against the
relayd.

The idea being that you can only start N sessions over
a given period of time.

*/

bstring
envtostr (char *vname)
{
  bstring retval;
  retval = bfromcstr (getenv (vname));
  if (!retval)
    retval = bfromcstr ("");

  //printf ("%s=%s\n",vname,retval->data);
  return retval;
}

int
main ()
{

  int ppid = getppid ();

  if (relayd_open () < 0)
    {
      fprintf (stderr, "ipthrottle: pid %d - Can't connect to relayd\n",
               ppid);
      return -1;
    }

  bstring remoteip = envtostr ("TCPREMOTEIP");

  if (remoteip->slen == 0)      //Should not happen
    {
      fprintf (stderr, "ipthrottle: pid %d - 511 No IP address\n", ppid);
      exit (0);
    }

  int result = relayd_check_ip (remoteip->data);

  if (result == 0)              // Exceeding current rate for IP
    {
      printf
        ("R421 Too many connections. Please reduce connections per minute.\n");
      fprintf (stderr, "ipthrottle: pid %d - Rate exceeded per IP (%s)\n",
               ppid, remoteip->data);
    }
  relayd_ack ("", remoteip->data);
  relayd_commit ();
  relayd_close ();
  exit (0);
}
