/*
* Copyright (C) 2006 Roberto Alsina <ralsina@kde.org>
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either
* version 2 of the License, or (at your option) any later
* version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software Foundation,
* Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*
*/

#include <stdio.h>
#include <stdlib.h>
#include "bstrlib.h"
#include <sys/types.h>
#include <unistd.h>

int
envtoint (char *vname, int defval)
{
  bstring envvar_;
  int retval = defval;

  envvar_ = bfromcstr (getenv (vname));
  if (envvar_)
    {
      retval = atoi (envvar_->data);
      bdestroy (envvar_);
    }
  //printf ("%s=%d\n",vname,retval);
  return retval;
}


int
main ()
{
  int ppid = getppid ();

  bstring cfname = bfromcstr (getenv ("LOCALMAILCONF"));
  if (!cfname)
    cfname = bfromcstr ("/var/qmail/control/localonly");

  FILE *cfile = fopen (cfname->data, "r");

  if (!cfile)
    {
      fprintf (stderr, "localmail: pid %d - error opening %s\n", ppid,
               cfname->data);
      exit (0);
    }

  bstring authuser = bfromcstr (getenv ("SMTPAUTHUSER"));
  if (!authuser)                //not auth. this plugin doesn't apply, exit quietly
    {
      exit (0);
    }
  bstring rcpt = bfromcstr (getenv ("SMTPRCPTTO"));
  if (!rcpt)                    //no rcpt?. Bad usage?
    {
      fprintf (stderr, "localmail: pid %d - error: no SMTPRCPTTO\n", ppid);
      exit (0);
    }

  bconchar (authuser, '\n');

  for (;;)
    {
      bstring user = bgets ((bNgetc) fgetc, cfile, '\n');
      if (!user)                //End of stream, so this must not be a localonly user
        exit (0);
      if (0 == bstrcmp (user, authuser))        //It's a localonly
        {
          if (0 == envtoint ("SMTPRCPTHOSTSOK", 0))     //recipient is remote
            {
              printf
                ("E550 You are only allowed to send to local addresses.");
              btrunc (authuser, authuser->slen - 1);
              fprintf (stderr, "localmail: pid %d - blocked from %s to %s\n",
                       ppid, authuser->data, rcpt->data);
            }
          exit (0);
        }
    }
  exit (0);
}
