/*
* Copyright (C) 2003-2004 Perolo Silantico <per.sil@gmx.it>
* Copyright (C) 2006 Roberto Alsina <ralsina@kde.org>
*
* For any questions please contact Roberto Alsina, because
* this version is modified from the original.
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either
* version 2 of the License, or (at your option) any later
* version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software Foundation,
* Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*
*/

#include <netinet/in.h>
#include <arpa/nameser.h>
#include <resolv.h>
#include <stdlib.h>
#include <errno.h>
#include <netdb.h>
#include "bstrlib.h"

int ppid;

void
block_permanent (const char *message)
{
  printf ("E553 sorry, %s (#5.7.1)\n", message);
  fprintf (stderr, "mfdnschecks: pid %d - blocked with: %s\n", ppid, message);
  exit (0);
}


void
block_temporary (const char *message)
{
  printf ("E451 %s (#4.3.0)\n", message);
  fprintf (stderr, "mfdnschecks: pid %d - temporary failure: %s\n", ppid,
           message);
  exit (0);
}

int
main (void)
{
  unsigned char dns_answer[1023];
  /* should be sufficient since we are not interested in the answer itself */
  bstring from = bfromcstr (getenv ("SMTPMAILFROM"));
  bstring buffer = NULL;

  ppid = getppid ();

  if (!from)
    {
      block_permanent ("no MAIL FROM envelope header has been sent.");
    }

  // Empty SMTPMAILFROM happens on bounces. The original plugin had them blocked,
  // but then you don't get bounces, and some servers will not even accept your
  // mail later. So, empty SMTPMAILFROM is A-OK.

  if (from->slen == 0)
    exit (0);

  // If it is not empty, but has no @ or no domain part or no username part, 
  // we don't like it.

  struct bstrList *pieces = bsplit (from, '@');

  if (pieces->qty != 2)
    {
      buffer =
        bformat ("invalid mail address in MAIL FROM envelope header: %s",
                 from->data);
      block_permanent (buffer->data);
    }
  bstring username = pieces->entry[0];
  bstring domain = pieces->entry[1];

  if (domain->slen == 0 || username->slen == 0)
    {
      buffer =
        bformat ("invalid mail address in MAIL FROM envelope header: %s",
                 from->data);
      block_permanent (buffer->data);
    }

  /* init DNS library */
  res_init ();

  /* make query */
  if (res_query (domain->data, C_IN, T_MX, dns_answer, sizeof (dns_answer)) <=
      0)
    {
      if ((errno == ECONNREFUSED) || (errno == TRY_AGAIN))
        block_temporary ("DNS temporary failure.");

      /* check for A record instead of MX record */
      else
        if (res_query
            (domain->data, C_IN, T_A, dns_answer, sizeof (dns_answer)) <= 0)
        {
          if ((errno == ECONNREFUSED) || (errno == TRY_AGAIN))
            block_temporary ("DNS temporary failure.");
          else
            buffer =
              bformat ("your envelope sender domain must exist: %s",
                       domain->data);
          block_permanent (buffer->data);
        }
    }
  fprintf (stderr, "mfdnschecks: pid %d - OK %s\n", ppid, from->data);
  exit (0);
}
